@@ -1,6 +1,6 @@
 /*
- *  6tunnel v0.13
- *  (C) Copyright 2000-2005,2013,2016,2019 by Wojtek Kaniewski <wojtekka@toxygen.net>
+ *  6tunnel
+ *  (C) Copyright 2000-2025 by Wojtek Kaniewski <wojtekka@toxygen.net>
  *
  *  Contributions by:
  *  - Dariusz Jackowski <ascent@linux.pl>
@@ -209,7 +209,6 @@ void make_tunnel(int rsock, const char *client_addr)
 {
 	char buf[4096], *outbuf = NULL, *inbuf = NULL;
 	int sock = -1, outlen = 0, inlen = 0;
-	struct sockaddr *sa = NULL;
 	const char *source;
 	struct addrinfo *connect_ai = NULL;
 	struct addrinfo *bind_ai = NULL;
@@ -469,6 +468,7 @@ void usage(const char *arg0)
 "  -s  connect using specified address\n"
 "  -m  read specified IPv4-to-IPv6 map file\n"
 "  -v  be verbose\n"
+"  -V  print version\n"
 "\n", arg0);
 }
 
@@ -557,7 +557,7 @@ void map_read(void)
 	fclose(f);
 }
 
-void sighup()
+void sighup(int unused)
 {
 	source_map_destroy();
 	map_read();
@@ -565,7 +565,7 @@ void sighup()
 	signal(SIGHUP, sighup);
 }
 
-void sigchld()
+void sigchld(int unused)
 {
 	while (waitpid(-1, NULL, WNOHANG) > 0) {
 		debug("child process exited\n");
@@ -575,7 +575,7 @@ void sigchld()
 	signal(SIGCHLD, sigchld);
 }
 
-void sigterm()
+void sigterm(int unused)
 {
 	if (pid_file != NULL)
 		unlink(pid_file);
@@ -586,19 +586,16 @@ void sigterm()
 int main(int argc, char **argv)
 {
 	int force = 0, listen_fd, single_connection = 0, jeden = 1, local_port;
-	int detach = 1, sa_len, conn_limit = 0, optc;
+	int detach = 1, conn_limit = 0, optc;
 	const char *username = NULL;
 	char *local_host = NULL;
 	struct addrinfo *ai;
 	struct addrinfo *ai_ptr;
-	struct sockaddr *sa;
-	struct sockaddr_in laddr;
-	struct sockaddr_in6 laddr6;
 	struct passwd *pw = NULL;
 	char *tmp;
 	int source_hint;
 
-	while ((optc = getopt(argc, argv, "1dv46fHs:l:I:i:hu:m:L:A:p:")) != -1) {
+	while ((optc = getopt(argc, argv, "1dv46fHs:l:I:i:hu:m:L:A:p:V")) != -1) {
 		switch (optc) {
 			case '1':
 				single_connection = 1;
@@ -651,6 +648,9 @@ int main(int argc, char **argv)
 			case 'H':
 				fprintf(stderr, "%s: warning: -H is deprecated, please use proper combination of -4 and -6.\n", argv[0]);
 				break;
+			case 'V':
+				printf("%s\n", PACKAGE_STRING);
+				exit(0);
 			default:
 				return 1;
 		}
@@ -848,8 +848,8 @@ int main(int argc, char **argv)
 		int client_fd;
 		char *client_addr;
 		int client_port;
-		struct sockaddr sa;
-		unsigned int sa_len = sizeof(sa);
+		struct sockaddr_in6 sin6;    // assume it's no smaller than sockaddr_in
+		unsigned int sin6_len = sizeof(sin6);
 
 		FD_ZERO(&rds);
 		FD_SET(listen_fd, &rds);
@@ -862,19 +862,18 @@ int main(int argc, char **argv)
 			break;
 		}
 
-		client_fd = accept(listen_fd, &sa, &sa_len);
+		client_fd = accept(listen_fd, (struct sockaddr*) &sin6, &sin6_len);
 
 		if (client_fd == -1) {
 			perror("accept");
 			break;
 		}
 
-		client_addr = xntop(&sa);
+		client_addr = xntop((struct sockaddr*) &sin6);
 
-		client_port = (sa.sa_family == AF_INET) ? ((struct sockaddr_in*) &sa)->sin_port :
-			((struct sockaddr_in6*) &sa)->sin6_port;
+		client_port = ntohs((sin6.sin6_family == AF_INET6) ? sin6.sin6_port : ((struct sockaddr_in*) &sin6)->sin_port);
 
-		debug("<%d> connection from %s,%d", client_fd, client_addr, ntohs(client_port));
+		debug("<%d> connection from %s,%d", client_fd, client_addr, client_port);
 
 		if (conn_limit && (conn_count >= conn_limit)) {
 			debug(" -- rejected due to limit.\n");
